#!/usr/bin/python3.6
"""Various tools to emerge and to show status for conky."""
from datetime import datetime
from re import search, sub

from pyerge import emerge_logfile, tmerge_logfile, tmplogfile


def e_sync() -> str:
    """
    Fetch date of last sync form logs.

    :return: date as string
    :rtype: str
    """
    with open(emerge_logfile) as log_file:
        for line in reversed(list(log_file)):
            match = search(r'(\d+)(:\s===\sSync completed)', line)
            if match is not None:
                sync_time = match.group(1)
                break
        else:
            return 'Unknown'
    return datetime.fromtimestamp(int(sync_time)).strftime('%A %H:%M')


def e_dl() -> str:
    """
    Fetch size of archives to be download for next system update.

    :return: date as string
    :rtype: str
    """
    with open(tmerge_logfile, 'r') as log_file:
        for line in reversed(list(log_file)):
            match = search(r'(Size of downloads:.)([0-9,]*\s[KMG]iB)', line)
            if match is not None:
                size = match.group(2)
                break
        else:
            size = None

    if size == '0 KiB':
        size = 'None'
    elif not size:
        size = 'Calculating...'

    return size


def e_curr() -> str:
    """
    Read the name of the last package compiled.

    :return: name of package with version
    :rtype: str
    """
    with open(emerge_logfile) as log_file:
        for line in reversed(list(log_file)):
            match = search(r'Compiling.*\((.*)::', line)
            if match is not None:
                pack = match.group(1)
                break
        else:
            pack = ''
    return pack


def e_eut() -> str:
    """
    Get estimated update time.

    :return: estimated update time
    :rtype: str
    """
    with open(tmplogfile) as log_file:
        for line in reversed(list(log_file)):
            match = search(r'Estimated update time:\s+(.*)\.', line)
            if match is not None:
                eut = match.group(1).replace(',', '')
                eut = sub(' minutes| minute', 'min', eut)
                eut = sub(' hours| hour', 'h', eut)
                break
        else:
            eut = 'Unknown'
    return eut



from pytest import fixture


@fixture
def str_no_sync():
    return """1548194358:  === sync
1548194358: >>> Syncing repository 'gentoo' into '/usr/portage'...
1548194361: >>> Starting rsync with rsync://[2a01:90:200:10::1a]/gentoo-portage"""


@fixture
def str_sync():
    return """1548194361: >>> Starting rsync with rsync://[2a01:90:200:10::1a]/gentoo-portage
1548194466: === Sync completed for gentoo
1548194466:  *** terminating."""


@fixture
def str_curr():
    return """1571086865:  *** emerge --nospinner --oneshot sys-kernel/linux-firmware
1571086944:  >>> emerge (1 of 1) sys-kernel/linux-firmware-20191008 to /
1571086944:  === (1 of 1) Cleaning (sys-kernel/linux-firmware-20191008::/usr/portage/sys-kernel/linux-firmware/linux-firmware-20191008.ebuild)
1571086985:  === (1 of 1) Compiling/Merging (sys-kernel/linux-firmware-20191008::/usr/portage/sys-kernel/linux-firmware/linux-firmware-20191008.ebuild)
1571086996:  === (1 of 1) Merging (sys-kernel/linux-firmware-20191008::/usr/portage/sys-kernel/linux-firmware/linux-firmware-20191008.ebuild)
1571087012:  >>> AUTOCLEAN: sys-kernel/linux-firmware:0
1571087012:  === Unmerging... (sys-kernel/linux-firmware-20191004)"""


@fixture
def str_curr_empty():
    return """1571086865:  *** emerge --nospinner --oneshot sys-kernel/linux-firmware
1571086944:  >>> emerge (1 of 1) sys-kernel/linux-firmware-20191008 to /
1571086944:  === (1 of 1) Cleaning (sys-kernel/linux-firmware-20191008::/usr/portage/sys-kernel/linux-firmware/linux-firmware-20191008.ebuild)
1571086996:  === (1 of 1) Merging (sys-kernel/linux-firmware-20191008::/usr/portage/sys-kernel/linux-firmware/linux-firmware-20191008.ebuild)
1571087012:  >>> AUTOCLEAN: sys-kernel/linux-firmware:0
1571087012:  === Unmerging... (sys-kernel/linux-firmware-20191004)"""


@fixture
def str_dl_gt_0():
    return """[ebuild     U  ] xfce-extra/thunar-archive-plugin-0.4.0::gentoo [0.3.1-r3::gentoo] 342 KiB
[ebuild     U  ] xfce-extra/thunar-shares-plugin-0.3.0::gentoo [0.2.0_p20101105-r1::gentoo] 333 KiB
[ebuild     U  ] xfce-base/xfce4-meta-4.14-r1::gentoo [4.12-r1::gentoo] USE="svg upower%* -minimal" 0 KiB

Total: 44 packages (42 upgrades, 2 new), Size of downloads: 283,699 KiB
"""

# todo: add correct printout
@fixture
def str_dl_eq_0():
    return """[ebuild     U  ] xfce-extra/thunar-archive-plugin-0.4.0::gentoo [0.3.1-r3::gentoo] 342 KiB
[ebuild     U  ] xfce-extra/thunar-shares-plugin-0.3.0::gentoo [0.2.0_p20101105-r1::gentoo] 333 KiB
[ebuild     U  ] xfce-base/xfce4-meta-4.14-r1::gentoo [4.12-r1::gentoo] USE="svg upower%* -minimal" 0 KiB

Total: 44 packages (42 upgrades, 2 new), Size of downloads: 0 KiB
"""

# todo: add correct printout
@fixture
def str_dl_unknown():
    return """[ebuild     U  ] xfce-extra/thunar-archive-plugin-0.4.0::gentoo [0.3.1-r3::gentoo] 342 KiB
[ebuild     U  ] xfce-extra/thunar-shares-plugin-0.3.0::gentoo [0.2.0_p20101105-r1::gentoo] 333 KiB
[ebuild     U  ] xfce-base/xfce4-meta-4.14-r1::gentoo [4.12-r1::gentoo] USE="svg upower%* -minimal" 0 KiB
"""


@fixture
def str_eut():
    return """[ebuild     U  ] xfce-base/xfdesktop-4.14.1::gentoo [4.12.4::gentoo] USE="libnotify thunar -debug" 1,498 KiB
[ebuild     U  ] xfce-extra/thunar-archive-plugin-0.4.0::gentoo [0.3.1-r3::gentoo] 342 KiB
[ebuild     U  ] xfce-extra/thunar-shares-plugin-0.3.0::gentoo [0.2.0_p20101105-r1::gentoo] 333 KiB
[ebuild     U  ] xfce-base/xfce4-meta-4.14-r1::gentoo [4.12-r1::gentoo] USE="svg upower%* -minimal" 0 KiB

!!! Error: couldn't get previous merge of libXpresent; skipping...

Estimated update time: 1 hour, 9 minutes.
"""


@fixture
def str_eut_unknown():
    return """[ebuild     U  ] xfce-base/xfdesktop-4.14.1::gentoo [4.12.4::gentoo] USE="libnotify thunar -debug" 1,498 KiB
[ebuild     U  ] xfce-extra/thunar-archive-plugin-0.4.0::gentoo [0.3.1-r3::gentoo] 342 KiB
[ebuild     U  ] xfce-extra/thunar-shares-plugin-0.3.0::gentoo [0.2.0_p20101105-r1::gentoo] 333 KiB
[ebuild     U  ] xfce-base/xfce4-meta-4.14-r1::gentoo [4.12-r1::gentoo] USE="svg upower%* -minimal" 0 KiB

!!! Error: couldn't get previous merge of libXpresent; skipping...
"""


@fixture
def orginal_html_xml():
    html = b'<?xml version="1.0" encoding="UTF-8"?>\n<rdf:RDF xmlns="http://purl.org/rss/1.0/"\n  ' \
           b'xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"\n  ' \
           b'xmlns:content="http://purl.org/rss/1.0/modules/content/"\n  ' \
           b'xmlns:dc="http://purl.org/dc/elements/1.1/"\n  ' \
           b'xmlns:image="http://purl.org/rss/1.0/modules/image/"\n  ' \
           b'xmlns:slash="http://purl.org/rss/1.0/modules/slash/"\n  ' \
           b'xmlns:sy="http://purl.org/rss/1.0/modules/syndication/"\n  ' \
           b'xmlns:taxo="http://purl.org/rss/1.0/modules/taxonomy/"\n  ' \
           b'xmlns:trackback="http://madskills.com/public/xml/rss/module/trackback/">\n' \
           b'<channel rdf:about="https://security.gentoo.org/glsa">\n    ' \
           b'<title>Gentoo Linux security advisories</title>\n    ' \
           b'<link>https://security.gentoo.org/glsa</link>\n    ' \
           b'<description>This feed contains new Gentoo Linux security advisories. ' \
           b'Contact security@gentoo.org with questions.</description>\n' \
           b'<items>\n      <rdf:Seq>\n' \
           b'<rdf:li resource="https://security.gentoo.org/glsa/201904-25"/>\n' \
           b'<rdf:li resource="https://security.gentoo.org/glsa/201904-24"/>\n' \
           b'<rdf:li resource="https://security.gentoo.org/glsa/201904-23"/>\n' \
           b'</items>\n    <dc:date>2019-04-24T00:00:00+00:00</dc:date>\n  </channel>\n' \
           b'<item rdf:about="https://security.gentoo.org/glsa/201904-25">\n    ' \
           b'<title>GLSA 201904-25: QEMU: Multiple vulnerabilities</title>\n    ' \
           b'<link>https://security.gentoo.org/glsa/201904-25</link>\n    ' \
           b'<dc:date>2019-04-24T00:00:00+00:00</dc:date>\n  </item>\n' \
           b'<item rdf:about="https://security.gentoo.org/glsa/201904-24">\n    ' \
           b'<title>GLSA 201904-24: Ming: Multiple vulnerabilities</title>\n    ' \
           b'<link>https://security.gentoo.org/glsa/201904-24</link>\n    ' \
           b'<dc:date>2019-04-24T00:00:00+00:00</dc:date>\n  </item>\n' \
           b'<item rdf:about="https://security.gentoo.org/glsa/201904-23">\n    ' \
           b'<title>GLSA 201904-23: GLib: Multiple vulnerabilities</title>\n    ' \
           b'<link>https://security.gentoo.org/glsa/201904-23</link>\n    ' \
           b'<dc:date>2019-04-22T00:00:00+00:00</dc:date>\n  </item>\n' \
           b'</rdf:RDF>'
    return html


@fixture()
def html_xlm_as_str():
    return """
<?xml version="1.0" encoding="UTF-8"?>
<rdf:RDF xmlns="http://purl.org/rss/1.0/"
  xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
  xmlns:content="http://purl.org/rss/1.0/modules/content/"
  xmlns:dc="http://purl.org/dc/elements/1.1/"
  xmlns:image="http://purl.org/rss/1.0/modules/image/"
  xmlns:slash="http://purl.org/rss/1.0/modules/slash/"
  xmlns:sy="http://purl.org/rss/1.0/modules/syndication/"
  xmlns:taxo="http://purl.org/rss/1.0/modules/taxonomy/"
  xmlns:trackback="http://madskills.com/public/xml/rss/module/trackback/">
  <channel rdf:about="https://security.gentoo.org/glsa">
    <title>Gentoo Linux security advisories</title>
    <link>https://security.gentoo.org/glsa</link>
    <description>This feed contains new Gentoo Linux security advisories. Contact security@gentoo.org with questions.</description>
    <items>
      <rdf:Seq>
        <rdf:li resource="https://security.gentoo.org/glsa/201904-23"/>
        <rdf:li resource="https://security.gentoo.org/glsa/201904-22"/>
      </rdf:Seq>
    </items>
    <dc:date>2019-04-24T00:00:00+00:00</dc:date>
  </channel>
  <item rdf:about="https://security.gentoo.org/glsa/201904-23">
    <title>GLSA 201904-23: GLib: Multiple vulnerabilities</title>
    <link>https://security.gentoo.org/glsa/201904-23</link>
    <dc:date>2019-04-22T00:00:00+00:00</dc:date>
  </item>
  <item rdf:about="https://security.gentoo.org/glsa/201904-22">
    <title>GLSA 201904-22: OpenDKIM: Root privilege escalation</title>
    <link>https://security.gentoo.org/glsa/201904-22</link>
    <dc:date>2019-04-22T00:00:00+00:00</dc:date>
  </item>
</rdf:RDF>
"""



from sys import version_info
from unittest import mock
from unittest.mock import mock_open, Mock

from pytest import mark

from pyerge.file_utils import e_sync, e_dl, e_curr, e_eut


def test_synced(str_sync):
    with mock.patch('pyerge.file_utils.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_sync.split('\n'))))
        assert e_sync() == 'Tuesday 22:01' or 'Tuesday 23:01'


def test_no_synced(str_no_sync):
    with mock.patch('pyerge.file_utils.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_no_sync.split('\n'))))
        assert e_sync() == 'Unknown'


@mark.skipif(condition=version_info < (3, 7), reason='Run only on Python 3.7+')
def test_synced_ver2(str_sync):
    with mock.patch('pyerge.file_utils.open', mock_open(read_data=str_sync)) as m:
        result = e_sync()

    m.assert_called_once_with('/var/log/emerge.log')
    assert result == 'Tuesday 22:01' or 'Tuesday 23:01'


def test_e_dl_gt_zero(str_dl_gt_0):
    with mock.patch('pyerge.file_utils.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_dl_gt_0.split('\n'))))
        assert e_dl() == '283,699 KiB'


def test_e_dl_eq_zero(str_dl_eq_0):
    with mock.patch('pyerge.file_utils.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_dl_eq_0.split('\n'))))
        assert e_dl() == 'None'


def test_e_dl_unknown(str_dl_unknown):
    with mock.patch('pyerge.file_utils.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_dl_unknown.split('\n'))))
        assert e_dl() == 'Calculating...'


def test_e_curr(str_curr):
    with mock.patch('pyerge.file_utils.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_curr.split('\n'))))
        assert e_curr() == 'sys-kernel/linux-firmware-20191008'


def test_e_curr_empty(str_curr_empty):
    with mock.patch('pyerge.file_utils.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_curr_empty.split('\n'))))
        assert e_curr() == ''


def test_e_eut(str_eut):
    with mock.patch('pyerge.file_utils.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_eut.split('\n'))))
        assert e_eut() == '1h 9min'


def test_e_eut_unknown(str_eut_unknown):
    with mock.patch('pyerge.file_utils.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_eut_unknown.split('\n'))))
        assert e_eut() == 'Unknown'
